"""SBOM Infrastructure Layer.

Infrastructure implementations for repositories, adapters, and external
integrations. This layer depends on the domain layer but the domain
is independent of infrastructure concerns.
"""
