//! Pipeline components for GraphQL response building
//!
//! This module contains the high-level pipeline for transforming
//! PostgreSQL JSON results into GraphQL HTTP responses.

pub mod builder;
pub mod projection;
