"""SBOM Domain Layer.

Contains the core domain model for Software Bill of Materials following
Domain-Driven Design principles. This layer is independent of infrastructure
concerns and represents the ubiquitous language of the supply chain domain.
"""
